'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.render = exports.retry = exports.spin = exports.prompt = undefined;

var _inquirer = require('inquirer');

var _inquirer2 = _interopRequireDefault(_inquirer);

var _when = require('when');

var _when2 = _interopRequireDefault(_when);

var _cliSpinner = require('cli-spinner');

require('./templates');

var _handlebars = require('handlebars');

var _handlebars2 = _interopRequireDefault(_handlebars);

var _log = require('../lib/log');

var _log2 = _interopRequireDefault(_log);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

_cliSpinner.Spinner.setDefaultSpinnerString(_cliSpinner.Spinner.spinners[7]);

function prompt(qs) {
	return _when2.default.promise(function promptPromise(resolve, reject) {
		if (!global.isInteractive) {
			return reject('prompts are not allowed in non-interactive mode');
		}

		_inquirer2.default.prompt(qs, function promptAnswers(answers) {
			resolve(answers);
		});
	});
}

function spin(promise, str) {
	if (!global.isInteractive) {
		return promise;
	}
	if (global.verboseLevel > 1) {
		_log2.default.debug(str);
		return promise;
	}

	const s = new _cliSpinner.Spinner(str);
	s.start();
	return (0, _when2.default)(promise).finally(() => s.stop(true));
}

function retry(fToRetry, times, handler, finalHandler) {
	if (!global.isInteractive) {
		times = 0;
	}

	return function retryFunc() {
		let tries = -1;
		const fContext = this;
		const args = Array.prototype.slice.call(arguments);

		function fAttempt(err) {
			tries++;
			if (tries >= times) {
				if (finalHandler) {
					return finalHandler(err);
				}
				return _when2.default.reject(err);
			}
			if (err) {
				handler(err);
			}
			return fToRetry.apply(fContext, args).catch(fAttempt);
		}
		return fAttempt();
	};
}

function render(templateName, data, supportingData) {
	if (global.outputJson) {
		process.stdout.write(JSON.stringify(data, null, 2));
		return;
	}
	process.stdout.write(_handlebars2.default.templates[templateName](Object.assign({ data }, supportingData)));
}

exports.prompt = prompt;
exports.spin = spin;
exports.retry = retry;
exports.render = render;