'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _when = require('when');

var _when2 = _interopRequireDefault(_when);

var _latestVersion = require('latest-version');

var _latestVersion2 = _interopRequireDefault(_latestVersion);

var _semver = require('semver');

var _semver2 = _interopRequireDefault(_semver);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _package = require('../../package');

var _package2 = _interopRequireDefault(_package);

var _settings = require('../../settings');

var _settings2 = _interopRequireDefault(_settings);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function spin() {
	return require('./ui').spin;
}

function check(skip, force) {
	return _when2.default.promise(resolve => {
		if (skip) {
			return resolve();
		}

		const now = Date.now();
		const lastCheck = _settings2.default.profile_json.last_version_check || 0;
		if (now - lastCheck >= _settings2.default.updateCheckInterval || force) {
			_settings2.default.profile_json.last_version_check = now;
			checkVersion().then(() => {
				_settings2.default.saveProfileData();
				start();
				resolve();
			});
			return;
		}

		resolve();
	});
}

function checkVersion() {
	const checkPromise = (0, _when2.default)((0, _latestVersion2.default)(_package2.default.name)).timeout(_settings2.default.updateCheckTimeout);

	return spin()(checkPromise, 'Checking for updates...').then(version => {
		if (_semver2.default.gt(version, _package2.default.version)) {
			_settings2.default.profile_json.newer_version = version;
		} else {
			delete _settings2.default.profile_json.newer_version;
		}
	}).catch(() => {});
}

function displayVersionBanner(version) {
	console.error('particle-cli v' + _package2.default.version);
	console.error();
	console.error(_chalk2.default.yellow('!'), 'A newer version (' + _chalk2.default.cyan(version) + ') of', _chalk2.default.bold.white('particle-cli'), 'is available.');
	console.error(_chalk2.default.yellow('!'), 'Upgrade now by running:', _chalk2.default.bold.white('npm install -g particle-cli'));
	console.error();
}

function start() {
	const storedVersion = _settings2.default.profile_json.newer_version;
	if (storedVersion && _semver2.default.gt(storedVersion, _package2.default.version)) {
		displayVersionBanner(storedVersion);
	}
}

exports.default = check;
module.exports = exports['default'];