'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

function api() {

	const ParticleApi = require('../cmd/api').default;
	const settings = require('../../settings');

	if (!api._instance) {
		api._instance = new ParticleApi(settings.apiUrl, {
			accessToken: settings.access_token
		}).api;
	}
	return api._instance;
}

exports.default = (_ref) => {
	let commandProcessor = _ref.commandProcessor,
	    root = _ref.root;

	const lib = commandProcessor.createCategory(root, 'library', 'Manage firmware libraries', { alias: 'libraries' });

	commandProcessor.createCommand(lib, 'add', 'Add a library to the current project.', {
		params: '<name>',
		handler: function () {
			var _require;

			for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
				args[_key] = arguments[_key];
			}

			return (_require = require('./library_add')).command.apply(_require, [api()].concat(args));
		},
		examples: {
			'$0 $command InternetButton': 'Add the InternetButton library to your project. Create a project with the project init command'
		}
	});

	commandProcessor.createCommand(lib, 'create', 'Create a new library in the specified or current directory', {
		options: {
			'name': {
				description: 'The name of the library to create.'
			},
			'version': {
				description: 'The initial version of the library to create.'
			},
			'author': {
				description: 'The author of the library.'
			}
		},
		handler: function () {
			var _require2;

			return (_require2 = require('./library_init')).command.apply(_require2, arguments);
		}
	});

	commandProcessor.createCommand(lib, 'install', false, {
		options: {
			'copy': {
				boolean: true,
				alias: 'vendored',
				description: 'install the library by copying the library sources into the project\'s lib folder'
			},
			'adapter': { // hidden
				boolean: true,
				alias: 'a'
			},
			'confirm': {
				boolean: true,
				alias: 'y'
			},
			'dest': {
				description: 'the directory to install to'
			}
		},
		params: '[name]',
		handler: function () {
			var _require3;

			for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
				args[_key2] = arguments[_key2];
			}

			return (_require3 = require('./library_install')).command.apply(_require3, ['install', api()].concat(args));
		}
	});

	commandProcessor.createCommand(lib, 'copy', 'Copy a library to the current project', {
		params: '[name]',
		handler: function () {
			var _require4;

			for (var _len3 = arguments.length, args = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
				args[_key3] = arguments[_key3];
			}

			return (_require4 = require('./library_install')).command.apply(_require4, ['copy', api()].concat(args));
		}
	});

	commandProcessor.createCommand(lib, 'list', 'List libraries available', {
		options: {
			'filter': {
				description: 'filters libraries not matching the text'
			},
			'non-interactive': {
				boolean: true,
				description: 'Prints a single page of libraries without prompting'
			},
			'page': {
				number: true,
				description: 'Start the listing at the given page number'
			},
			'limit': {
				number: true,
				description: 'The number of items to show per page'
			}
		},
		params: '[sections...]',
		handler: function () {
			var _require5;

			for (var _len4 = arguments.length, args = Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
				args[_key4] = arguments[_key4];
			}

			return (_require5 = require('./library_list')).command.apply(_require5, [api()].concat(args));
		}
	});

	commandProcessor.createCommand(lib, 'migrate', 'Migrate a local library from v1 to v2 format', {
		options: {
			test: {
				alias: 'dryrun',
				boolean: true,
				description: 'test if the library can be migrated'
			},
			'adapter': {
				boolean: true,
				default: true,
				description: 'add include file adapters to support v1-style includes "library/library.h"'
			}
		},
		params: '[library...]',

		handler: function () {
			var _require6;

			return (_require6 = require('./library_migrate')).command.apply(_require6, arguments);
		}
	});

	commandProcessor.createCommand(lib, 'search', 'Search available libraries', {
		params: '<name>',
		handler: function () {
			var _require7;

			for (var _len5 = arguments.length, args = Array(_len5), _key5 = 0; _key5 < _len5; _key5++) {
				args[_key5] = arguments[_key5];
			}

			return (_require7 = require('./library_search')).command.apply(_require7, [api()].concat(args));
		}
	});

	commandProcessor.createCommand(lib, 'upload', 'Uploads a private version of a library.', {
		options: {
			'dryRun': {
				boolean: true,
				description: 'perform validation steps but don\'t actually upload the library.'
			}
		},
		handler: function () {
			var _require8;

			for (var _len6 = arguments.length, args = Array(_len6), _key6 = 0; _key6 < _len6; _key6++) {
				args[_key6] = arguments[_key6];
			}

			return (_require8 = require('./library_upload')).command.apply(_require8, [api()].concat(args));
		}
	});

	commandProcessor.createCommand(lib, 'publish', 'Publish a private library, making it public', {
		params: '[name]',
		handler: function () {
			var _require9;

			for (var _len7 = arguments.length, args = Array(_len7), _key7 = 0; _key7 < _len7; _key7++) {
				args[_key7] = arguments[_key7];
			}

			return (_require9 = require('./library_publish')).command.apply(_require9, [api()].concat(args));
		}
	});

	commandProcessor.createCommand(lib, 'view', 'View details about a library', {
		options: {
			'readme': {
				boolean: true,
				description: 'display the readme for the library'
			},
			'source': {
				boolean: true,
				description: 'display the main source file for the library'
			},
			'header': {
				boolean: true,
				description: 'display the main header file for the library'
			}

		},
		params: '<name>',
		handler: function () {
			var _require10;

			for (var _len8 = arguments.length, args = Array(_len8), _key8 = 0; _key8 < _len8; _key8++) {
				args[_key8] = arguments[_key8];
			}

			return (_require10 = require('./library_view')).command.apply(_require10, [api()].concat(args));
		}
	});

	commandProcessor.createCommand(lib, 'delete', false, {
		params: '<name>',
		handler: function () {
			var _require11;

			for (var _len9 = arguments.length, args = Array(_len9), _key9 = 0; _key9 < _len9; _key9++) {
				args[_key9] = arguments[_key9];
			}

			return (_require11 = require('./library_delete')).command.apply(_require11, [api()].concat(args));
		}
	});

	return lib;
};

module.exports = exports['default'];