'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.CLILibraryInstallCommandSite = undefined;
exports.command = command;

var _cmd = require('../cmd');

var _api = require('../cmd/api');

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _apiclient = require('./apiclient');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class CLILibraryInstallCommandSite extends _cmd.LibraryInstallCommandSite {

	constructor(argv, dir, apiClient) {
		super();
		this._apiClient = apiClient;
		this.argv = argv;
		this.dir = dir;
	}

	apiClient() {
		return this._apiClient;
	}

	isVendored() {
		return this.argv.vendored;
	}

	isAdaptersRequired() {
		return this.argv.adapter;
	}

	libraryName() {
		const params = this.argv.params;
		return params && params.name;
	}

	targetDirectory() {
		return this.dir;
	}

	homePathOverride() {
		return this.argv.dest;
	}

	error(error) {
		throw (0, _api.convertApiError)(error);
	}

	notifyIncorrectLayout(actualLayout, expectedLayout, libName, targetDir) {
		return this.promiseLog(`Cannot install library: directory '${targetDir}' is a '${actualLayout}' format project, please change to a '${expectedLayout}' format.`);
	}

	notifyCheckingLibrary(libName) {
		return this.promiseLog(`Checking library ${_chalk2.default.green(libName)}...`);
	}

	notifyFetchingLibrary(lib, targetDir) {
		const dest = ` to ${targetDir}`;
		return this.promiseLog(`Installing library ${_chalk2.default.blue(lib.name)} ${lib.version}${dest} ...`);
	}

	notifyInstalledLibrary(lib, targetDir) {
		return this.promiseLog(`Library ${_chalk2.default.blue(lib.name)} ${lib.version} installed.`);
	}

	promiseLog(msg) {
		return Promise.resolve().then(() => console.log(msg));
	}
}

exports.CLILibraryInstallCommandSite = CLILibraryInstallCommandSite;
function install(argv, apiJS) {
	const site = new CLILibraryInstallCommandSite(argv, process.cwd(), (0, _apiclient.buildAPIClient)(apiJS));
	const cmd = new _cmd.LibraryInstallCommand();
	return site.run(cmd);
}

function copy(argv, apiJS) {
	argv.vendored = true;
	argv.adapter = false;
	argv.confirm = false;
	return install(argv, apiJS);
}

function command(cmd, apiJS, argv) {
	if (cmd === 'copy') {
		return copy(argv, apiJS);
	} else if (cmd === 'install') {
		return install(argv, apiJS);
	} else {
		throw Error('uknown command ' + cmd);
	}
}