'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.CLILibraryListCommandSite = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

exports.command = command;

var _cmd = require('../cmd');

var _api = require('../cmd/api');

var _ui = require('../app/ui');

var _apiclient = require('./apiclient');

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _library_ui = require('./library_ui.js');

var _prompts = require('../../dist/lib/prompts');

var _prompts2 = _interopRequireDefault(_prompts);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class CLILibraryListCommandSite extends _cmd.LibraryListCommandSite {

	constructor(argv, apiClient) {
		super();
		this._apiClient = apiClient;
		this.argv = argv;
		let sections = argv.params.sections;
		if (!sections || !sections.length) {
			sections = ['mine', 'community'];
		}
		this._sectionNames = sections;
		this._page = this.argv.page || 1;

		// todo - since the text can be used by any app, this could be pushed down to the command layer so it's shared
		this.headings = {
			official: 'Official Libraries',
			verified: 'Verified Libraries',
			popular: 'Popular Libraries',
			mine: 'My Libraries',
			recent: 'Recently added/updated Libraries',
			community: 'Community Libraries'
		};

		this._sections = this._buildSections();
	}

	apiClient() {
		return this._apiClient;
	}

	error(error) {
		throw (0, _api.convertApiError)(error);
	}

	target() {
		return {};
	}

	/**
  * Use the default settings
  * @returns {Object} Empty object {}
  */
	settings() {
		const result = {};
		this._add(result, 'filter');
		this._add(result, 'limit');
		return result;
	}

	_add(target, param) {
		if (this.argv[param]) {
			target[param] = this.argv[param];
		}
	}

	sectionNames() {
		return this._sectionNames;
	}

	sections() {
		return this._sections;
	}

	_buildSections() {
		const result = {};
		const sections = this.sectionNames();
		for (let section of sections) {
			result[section] = { page: 1 };
		}
		if (result.mine) {
			result.mine.excludeBadges = { mine: true };
		}
		return result;
	}

	_nextPage() {
		this._page++;
		for (let name of this.sectionNames()) {
			const section = this._sections[name];
			section.page += 1;
		}
	}

	_removeEmptySections(results) {
		let sectionNames = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this._sectionNames;
		let sections = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : this._sections;

		for (let name of sectionNames) {
			const section = results[name];
			if (!section || !section.length) {
				this._removeSection(name, sectionNames, sections);
			}
		}
		return [sectionNames, sections];
	}

	_removeSection(name) {
		let sectionNames = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this._sectionNames;
		let sections = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : this._sections;

		sectionNames.splice(sectionNames.indexOf(name), 1);
		delete sections[name];
	}

	notifyFetchLists(promise) {
		const msg = this._page === 1 ? 'Searching for libraries...' : `Retrieving libraries page ${this._page}`;
		return (0, _ui.spin)(promise, msg).then(results => {
			const sections = this.sectionNames();
			let separator = false;
			for (let name of sections) {
				const list = results[name];
				if (list) {
					if (separator) {
						console.log();
					}
					this.printSection(name, this._sections[name], list);
					separator = true;
				}
			}
			return results;
		});
	}

	printSection(name, section, libraries) {
		// omit the section if not on the first page and there are no results
		if (!libraries.length && section.page) {
			return;
		}

		const page = section.page > 1 ? _chalk2.default.grey(` page ${section.page}`) : '';
		const heading = this.headings[name] || name;
		console.log(_chalk2.default.bold(heading) + page);
		if (libraries.length) {
			for (let library of libraries) {
				this.showLibrary(section, library);
			}
		} else {
			console.log(_chalk2.default.grey('No libraries to show in this section.'));
		}
	}

	showLibrary(section, library) {
		console.log((0, _library_ui.formatLibrary)(library, section.excludeBadges));
	}
}

exports.CLILibraryListCommandSite = CLILibraryListCommandSite;
function command(apiJS, argv) {
	const site = new CLILibraryListCommandSite(argv, (0, _apiclient.buildAPIClient)(apiJS));
	const cmd = new _cmd.LibraryListCommand();

	function prompForNextPage() {
		return _prompts2.default.enterToContinueControlCToExit();
	}

	function runPage() {
		return site.run(cmd).then(results => nextPage(results));
	}

	function nextPage(results) {
		if (results) {
			// only continue to show sections with results;
			var _site$_removeEmptySec = site._removeEmptySections(results),
			    _site$_removeEmptySec2 = _slicedToArray(_site$_removeEmptySec, 1);

			const names = _site$_removeEmptySec2[0];

			if (!argv['non-interactive'] && names.length) {
				site._nextPage();
				return prompForNextPage().then(next => {
					if (next) {
						return runPage();
					}
				});
			}
		}
	}

	return runPage();
}