'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.CLILibraryMigrateCommandSite = exports.CLILibraryTestMigrateCommandSite = exports.CLIBaseLibraryMigrateCommandSite = undefined;
exports.command = command;

var _cmd = require('../cmd');

class CLIBaseLibraryMigrateCommandSite extends _cmd.LibraryMigrateCommandSite {
	constructor(argv, defaultDir) {
		super();
		this.argv = argv;
		if (!argv.params.library || !argv.params.library.length) {
			argv.params.library = [defaultDir];
			this.cwd = true;
		}
		this.libraries = argv.params.library;
		this.result = null;
	}

	isAdaptersRequired() {
		return this.argv.adapter;
	}

	getLibraries() {
		return this.libraries;
	}

	notifyEnd(lib, data, err) {
		this.result = { lib, data, err };
	}

	handleError(lib, err) {
		if (err.name === 'LibraryNotFoundError') {
			if (this.cwd) {
				console.error('No valid library found in current directory');
			} else {
				console.error('No valid library found in ' + lib);
			}
		} else {
			if (this.cwd) {
				console.error(`Error processing library in current directory: ${err}`);
			} else {
				console.error(`Error processing library '${lib}': ${err}`);
			}
		}
	}
}

exports.CLIBaseLibraryMigrateCommandSite = CLIBaseLibraryMigrateCommandSite;
class CLILibraryTestMigrateCommandSite extends CLIBaseLibraryMigrateCommandSite {

	notifyEnd(lib, result, err) {
		super.notifyEnd(lib, result, err);
		if (err) {
			this.handleError(lib, err);
		} else {
			if (result === 1) {
				if (this.cwd) {
					console.info('Library can be migrated');
				} else {
					console.info(`Library can be migrated: '${lib}'`);
				}
			} else {
				if (this.cwd) {
					console.info('Library already in v2 format');
				} else {
					console.info(`Library already in v2 format: '${lib}'`);
				}
			}
		}
	}
}

exports.CLILibraryTestMigrateCommandSite = CLILibraryTestMigrateCommandSite;
class CLILibraryMigrateCommandSite extends CLIBaseLibraryMigrateCommandSite {
	notifyEnd(lib, result, err) {
		if (err) {
			this.handleError(lib, err);
		} else {
			if (result === true) {
				console.info(`Library migrated to v2 format: '${lib}'`);
			} else {
				console.info(`Library already in v2 format: '${lib}'`);
			}
		}
	}
}

exports.CLILibraryMigrateCommandSite = CLILibraryMigrateCommandSite;
function command(argv) {
	let Site, Cmd;
	if (argv.test) {
		Site = CLILibraryTestMigrateCommandSite;
		Cmd = _cmd.LibraryMigrateTestCommand;
	} else {
		Site = CLILibraryMigrateCommandSite;
		Cmd = _cmd.LibraryMigrateCommand;
	}
	const site = new Site(argv, process.cwd());
	const cmd = new Cmd();
	return site.run(cmd);
}