'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.CLILibraryPublishCommandSite = undefined;
exports.command = command;

var _cmd = require('../cmd');

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _log = require('../lib/log');

var _log2 = _interopRequireDefault(_log);

var _ui = require('../app/ui');

var _apiclient = require('./apiclient');

var _library_upload = require('./library_upload');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class CLILibraryPublishCommandSite extends _cmd.LibraryPublishCommandSite {

	constructor(argv, apiClient) {
		super();
		this.argv = argv;
		this.ident = argv.params.name;
		this._apiClient = apiClient;
	}

	libraryIdent() {
		return this.ident;
	}

	libraryDirectory() {
		return this.dir;
	}

	apiClient() {
		return this._apiClient;
	}

	error(error) {
		throw error;
	}

	publishingLibrary(promise, ident) {
		return (0, _ui.spin)(promise, `Publishing library ${_chalk2.default.green(ident)}`);
	}

	publishLibraryComplete(library) {
		return _log2.default.success(`Library ${_chalk2.default.green(library.name)} was successfully published.`);
	}
}

exports.CLILibraryPublishCommandSite = CLILibraryPublishCommandSite;
class CLILibraryPublishContributeCommandSite extends _library_upload.CLILibraryContributeCommandSite {

	/**
  * Saves the constributed library and doesn't output a contributed success message since
  * the publish steps comes immediately afterwards - only want to print success when all steps
  * are complete.
  * @param {Library} library   The library that was contributed.
  */
	contributeComplete(library) {
		this.contributedLibrary = library;
	}
}

function command(apiJS, argv) {
	const site = new CLILibraryPublishCommandSite(argv, (0, _apiclient.buildAPIClient)(apiJS));
	const cmd = new _cmd.LibraryPublishCommand();
	let promise = Promise.resolve();
	if (!site.libraryIdent()) {
		// no library name given - try publishing the current library
		const contributeSite = new CLILibraryPublishContributeCommandSite(argv, process.cwd(), (0, _apiclient.buildAPIClient)(apiJS));
		// todo - set more stringent validation on the contribute command since this is pre-publish
		const contribute = new _cmd.LibraryContributeCommand();
		promise = contributeSite.run(contribute).then(() => {
			site.ident = contributeSite.contributedLibrary.name;
		});
	}
	return promise.then(() => site.run(cmd));
}