'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.command = command;

var _library_install = require('./library_install');

var _cmd = require('../cmd');

var _apiclient = require('./apiclient');

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class CLILibraryViewCommandSite extends _library_install.CLILibraryInstallCommandSite {

	notifyFetchingLibrary(lib, targetDir) {

		const targetExists = _fs2.default.existsSync(targetDir);

		if (!this.targetDir) {
			this.targetDir = targetDir;
			this.targetExists = targetExists;
		}

		return Promise.resolve(!targetExists);
	}

	notifyInstalledLibrary(lib, targetDir) {
		if (!this.metadata) {
			this.metadata = lib;
		}
		return super.notifyInstalledLibrary(lib, targetDir);
	}

	view() {
		if (this.argv.readme) {
			this.showFile('No readme files found for the library.', ['README.md', 'README.txt']);
		}

		if (this.argv.source) {
			this.showFile('No source files found for the library.', [_path2.default.join('src', this.metadata.name + '.cpp')]);
		}

		if (this.argv.header) {
			this.showFile('No header files found for the library.', [_path2.default.join('src', this.metadata.name + '.h')]);
		}

		console.log(`To view the library documentation and sources directly, please change to the directory ${_chalk2.default.bold(this.targetDir)}`);
	}

	showFile(missing, files) {
		let shown = false;
		for (let file of files) {
			const content = this.loadFile(file);
			if (content !== undefined) {
				console.log(content);
				shown = true;
				break;
			}
		}

		if (!shown) {
			console.log(missing);
		}
	}

	loadFile(file) {
		const full = _path2.default.join(this.targetDir, file);
		try {
			return _fs2.default.readFileSync(full, 'utf-8');
		} catch (error) {
			return undefined;
		}
	}
}

function command(apiJS, argv) {
	const site = new CLILibraryViewCommandSite(argv, process.cwd(), (0, _apiclient.buildAPIClient)(apiJS));
	const cmd = new _cmd.LibraryInstallCommand();
	return site.run(cmd).then(() => site.view());
}