'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _stream = require('stream');

var _stream2 = _interopRequireDefault(_stream);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class FilteredLogStream extends _stream2.default.Transform {
	constructor() {
		super();
	}

	_transform(data, encoding, callback) {
		if (global.verboseLevel < 2) {
			return callback();
		}
		this.push(data, encoding);
		callback();
	}
}

exports.default = {
	silly() {
		var _console;

		if (global.verboseLevel < 4) {
			return;
		}
		(_console = console).log.apply(_console, arguments);
	},

	verbose() {
		var _console2;

		if (global.verboseLevel < 3) {
			return;
		}
		(_console2 = console).log.apply(_console2, arguments);
	},

	debug() {
		var _console3;

		if (global.verboseLevel < 2) {
			return;
		}
		(_console3 = console).log.apply(_console3, arguments);
	},

	info() {
		var _console4;

		if (global.verboseLevel < 1) {
			return;
		}
		(_console4 = console).log.apply(_console4, arguments);
	},

	warn() {
		var _console5;

		if (global.verboseLevel < 1) {
			return;
		}
		(_console5 = console).error.apply(_console5, [_chalk2.default.yellow('!')].concat(Array.prototype.slice.call(arguments)));
	},

	success() {
		var _console6;

		if (global.verboseLevel < 1) {
			return;
		}
		(_console6 = console).log.apply(_console6, [_chalk2.default.green('>')].concat(Array.prototype.slice.call(arguments)));
	},

	error() {
		var _console7;

		if (global.verboseLevel < 1) {
			return;
		}
		(_console7 = console).error.apply(_console7, [_chalk2.default.red('!')].concat(Array.prototype.slice.call(arguments)));
	},

	fatal() {
		if (global.verboseLevel < 1) {
			return;
		}
		console.error(_chalk2.default.red.apply(_chalk2.default, arguments));
	},

	serialInput(data) {
		if (global.verboseLevel < 3) {
			return;
		}
		const lines = data.split('\n');
		lines.forEach(l => {
			if (!l) {
				return;
			}
			console.log(_chalk2.default.gray('Serial In <-'), l);
		});
	},

	serialOutput(data) {
		if (global.verboseLevel < 3) {
			return;
		}
		const lines = data.split('\n');
		lines.forEach(l => {
			if (!l) {
				return;
			}
			console.log(_chalk2.default.gray('Serial Out ->'), l);
		});
	},

	stdout() {
		const outStream = new FilteredLogStream();
		outStream.pipe(process.stdout);
		return outStream;
	},
	stderr() {
		const errStream = new FilteredLogStream();
		errStream.pipe(process.stderr);
		return errStream;
	}
};
module.exports = exports['default'];